;*****************************************************************************
; Copyright (c) 2013-2014, Texas Instruments Incorporated - http://www.ti.com/
;   All rights reserved.
;
;   Redistribution and use in source and binary forms, with or without
;   modification, are permitted provided that the following conditions are met:
;       * Redistributions of source code must retain the above copyright
;         notice, this list of conditions and the following disclaimer.
;       * Redistributions in binary form must reproduce the above copyright
;         notice, this list of conditions and the following disclaimer in the
;         documentation and/or other materials provided with the distribution.
;       * Neither the name of Texas Instruments Incorporated nor the
;         names of its contributors may be used to endorse or promote products
;         derived from this software without specific prior written permission.
;
;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
;   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
;   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
;   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
;   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
;   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
;   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
;   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
;   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
;   THE POSSIBILITY OF SUCH DAMAGE.
;****************************************************************************/

;**************************************************************************
; Given a buffer with the following structure:
;
;     4      4     4    ArgSz1   4    ArgSz2       4
; +-------+-----+-----+-------+-----+-------+---+-----+
; |FcnAddr| DP  |ArgSz|ArgData|ArgSz|ArgData|...|  0  |
; +-------+-----+-----+-------+-----+-------+---+-----+
;
;  This routine will branch to the FcnAddr after setting up the argument
;  registers based on the Arg data in the buffer.
; 
; After first 10 arguments, additional arguments are laid out in memory
; in the order of calling convention.  Copy more args from memory onto stack
; frame before calling child, roll back more args on stack frame after call.
; For example, starting from 11-th argument, memory layout:
;   4   4      4        4        8
; +---+---+--------+---------+--------+-----+-------+---+-----+
; |pad|int|char,pad|short,pad| double | ...
; +---+---+--------+---------+--------+-----+-------+---+-----+
; more args block size are rounded up to multiple of 8 bytes.
;
;**************************************************************************

	.asg B15, SP
	.asg B14, DP


ARG_HANDLER .macro SS1, SS2	
	 LDW 	*A0++, A1
	 NOP 	4
   [!A1] BNOP 	__MAKE_CALL, 5
         SUB 	A1, 4, A1
   [!A1] LDW 	*A0++, SS1
   [ A1] LDNDW 	*A0++, SS2:SS1
	 .endm

ARG_HANDLER_LAST .macro SS1, SS2	
	 ARG_HANDLER SS1, SS2
   	 ADD	4, A0, A0	; in order to advance past null terminator
	 .endm

	.if __TI_EABI__
	.global dsp_rpc
dsp_rpc:
	.else
	.global _dsp_rpc
_dsp_rpc:
	.endif

    STW     A10,*SP--(48)
||  MV      A4, A0            ; Save input message pointer

    ADD     B4, A6, B2        ; save more args (after first 10) end address
||  STW     A14, *+SP(44)     ; preserve parent's A14 on stack
    MV      A6, A14           ; save more args size, A14 persists across child
||  MV      A6, B0            ; copy more args size to loop counter

    STW     A11,*+SP(4)
    STDW    B11:B10,*+SP(8)
||  LDW     *A0++, A2         ; Get function address
    STDW    A13:A12,*+SP(16)
||  LDW     *A0++, B1         ; Get Data Page Pointer
    STDW    B13:B12,*+SP(24)

    STW     A0, *+SP(32)
    STW     B3, *+SP(36)
    STW     DP, *+SP(40)

    ARG_HANDLER 	A4, A5
    ARG_HANDLER 	B4, B5
    ARG_HANDLER 	A6, A7
    ARG_HANDLER 	B6, B7
    ARG_HANDLER 	A8, A9
    ARG_HANDLER 	B8, B9
    ARG_HANDLER 	A10, A11
    ARG_HANDLER 	B10, B11
    ARG_HANDLER 	A12, A13
    ARG_HANDLER_LAST 	B12, B13

    [!B0] BNOP 	__MAKE_CALL, 5
__COPY_MORE_ARGS:             ; copy from back to front
    LDDW    *--B2, A1:A0      ; copy from more args in memory
||  SUB     B0, 8, B0
    [ B0] BNOP  __COPY_MORE_ARGS, 4
    STDW    A1:A0, *--SP      ; copy onto stack frame for child call

__MAKE_CALL:
    CALL 	A2
    NOP         3
    MVKL 	__TI_OCL_WG_RETURN, B3
||  MV          B1, DP

__TI_OCL_WG_LAUNCH:
    MVKH 	__TI_OCL_WG_RETURN, B3

__TI_OCL_WG_RETURN:

    ADD         SP, A14, SP    ; roll back more args on stack frame
||  LDW		*+SP(44), A14  ; restore parent's A14

    LDW		*+SP(36), B3
    LDW		*+SP(40), DP
    LDW		*+SP(32), A4
    LDDW    	*+SP(24), B13:B12
    LDDW    	*+SP(16), A13:A12
    LDDW    	*+SP(8),  B11:B10
    LDW     	*+SP(4),  A11
    RET 	B3
    LDW     	*++SP(48), A10
    NOP         4

	.global __TI_OCL_WG_RETURN
	.global __TI_OCL_WG_LAUNCH
