#include <xdc/std.h>
#include <string.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/family/c66/Cache.h>
#include <ti/sdo/fc/edmamgr/edmamgr.h>

//#define FC_TRACE
#ifdef FC_TRACE
#include <ti/sdo/fc/global/FCSettings.h>
#include <xdc/runtime/Diags.h>
#endif

#include "stdio.h"
#include "monitor.h"

#define ADDR_IS_EDMA3_COHERENT(addr) ((addr) < (void*)0x80000000)

extern cregister volatile unsigned int DNUM;

#define BUF_SIZE (1<<19)
DDR_1D(uint8_t, input,  BUF_SIZE);
DDR_1D(uint8_t, output, BUF_SIZE);

/******************************************************************************
* initialize_edmamgr
******************************************************************************/
int initialize_edmamgr()
{
#ifdef FC_TRACE
   FCSettings_init();
   Diags_setMask(FCSETTINGS_MODNAME"+EX1234567");
#endif

   return !EdmaMgr_init(DNUM, NULL);
}

/******************************************************************************
* edmamgr_copy
******************************************************************************/
edmamgr_copy(void* dst, void* src, unsigned int bytes)
{
   EdmaMgr_Handle ev = EdmaMgr_alloc(1);

   if (!ev) 
   { 
       memcpy(dst, src, bytes); 
       printf("Failed to edma handle. memcpy fallback taken!\n");
       return; 
   }
 
   if (!ADDR_IS_EDMA3_COHERENT(src)) Cache_wb(src, bytes, Cache_Type_ALL, TRUE);
   EdmaMgr_copy1D1D(ev, src, dst, bytes);
   EdmaMgr_wait(ev);
   if (!ADDR_IS_EDMA3_COHERENT(dst)) Cache_inv(dst, bytes, Cache_Type_ALL, TRUE);
   EdmaMgr_free(ev);
}

/******************************************************************************
* void edma_test
******************************************************************************/
void edma_test()
{
   char* in  = (char*)&input[(BUF_SIZE/8) * DNUM];
   char* out = (char*)&output[(BUF_SIZE/8) * DNUM];

   char val = 10;
   int buf_size;
   for (buf_size = 256; buf_size <= (1<<15); buf_size <<= 1)
   {
       memset((void*)in, val++, buf_size);
       memset((void*)out, 0,    buf_size);

       edmamgr_copy((void*)out, (void*)in, buf_size);

       if (memcmp((void*)out, (void*)in, buf_size)) 
           printf("Failed (sz = %d)\n", buf_size);
   }
}
