/******************************************************************************
 * Copyright (c) 2013-2014, Texas Instruments Incorporated - http://www.ti.com/
 *   All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *       * Redistributions of source code must retain the above copyright
 *         notice, this list of conditions and the following disclaimer.
 *       * Redistributions in binary form must reproduce the above copyright
 *         notice, this list of conditions and the following disclaimer in the
 *         documentation and/or other materials provided with the distribution.
 *       * Neither the name of Texas Instruments Incorporated nor the
 *         names of its contributors may be used to endorse or promote products
 *         derived from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "monitor.h"
#include "ti_em_init.h"
#include "ocl_em_init.h"
#include <ti/csl/csl_cache.h>
#include <stdlib.h>
#include <ti/runtime/openmp/src/tomp_qmss_api.h>

/******************************************************************************
* Workgroup event buffer area
******************************************************************************/
DDR_1D(uint8_t, em_mem_buf_workgroup, EM_MAX_WG_EVENTS * EM_EVENT_WG_BUF_SIZE);
FAST_SHARED_1D(uint8_t, em_mem_buf_notify, EM_MAX_NOTIFY_EVENTS * EM_EVENT_NOTIFY_BUF_SIZE);

FAST_SHARED(em_eo_t,          EM_EO_workgroup);
FAST_SHARED(em_eo_t,          EM_EO_kernel_complete);
FAST_SHARED(em_eo_t,          EM_EO_exit);
FAST_SHARED(em_queue_t,       EM_Q_workgroup);
FAST_SHARED(em_queue_t,       EM_Q_exit);
FAST_SHARED(em_queue_group_t, EM_QGRP_wg);
FAST_SHARED(em_queue_group_t, EM_QGRP_c0);

FAST_SHARED_1D(em_queue_t,       EM_Q_notify, NUM_CORES);
FAST_SHARED_1D(em_queue_group_t, EM_QGRP_core, NUM_CORES);
FAST_SHARED_1D(em_event_group_t, em_hdl_event_group,  EM_EVENT_GROUP_NUM);
FAST_SHARED_2D(em_event_t, em_hdl_event_notify, EM_EVENT_GROUP_NUM, NUM_CORES);
FAST_SHARED_2D(em_notif_t, em_hdl_notify_desc,  EM_EVENT_GROUP_NUM, NUM_CORES);

uint32_t eventGroupIdx;

/******************************************************************************
* em_initPoolConfig
******************************************************************************/
static void em_initPoolConfig(ti_em_pool_config2_t* poolCfg)
{
#ifdef TI_EM_C6678
    static const int free_queue_1st = MY_EM_FREE_QUEUE_BASE_IDX;  // 2048
#endif
#ifdef TI_EM_C6634
    static const int free_queue_1st = MY_EM_FREE_QUEUE_BASE_IDX;  // 6802
#endif
    ti_em_pool_config2_t* poolPtr;

    poolPtr = &poolCfg[EM_POOL_NOTIFY_IDX];
    poolPtr->pool_config.free_queue_idx   = free_queue_1st + EM_POOL_NOTIFY_IDX;
    poolPtr->pool_config.buf_mode         = TI_EM_BUF_MODE_TIGHT;
    poolPtr->pool_config.dsc_wsize        = TI_EM_PUBLIC_EVENT_DSC_SIZE;
    poolPtr->event_num                    = EM_MAX_NOTIFY_EVENTS;
    poolPtr->free_policy                  = 0;
    poolPtr->buffer_ptr                   = em_mem_buf_notify;
    poolPtr->pool_config.buf_size         = EM_EVENT_NOTIFY_BUF_SIZE;
    poolPtr->coh_mode                     = TI_EM_COH_MODE_OFF;

    poolPtr = &poolCfg[EM_POOL_WG_IDX];
    poolPtr->pool_config.free_queue_idx   = free_queue_1st + EM_POOL_WG_IDX;
    poolPtr->pool_config.buf_mode         = TI_EM_BUF_MODE_TIGHT;
    poolPtr->pool_config.dsc_wsize        = TI_EM_PUBLIC_EVENT_DSC_SIZE;
    poolPtr->event_num                    = EM_MAX_WG_EVENTS;
    poolPtr->free_policy                  = 0;
    poolPtr->buffer_ptr                   = em_mem_buf_workgroup;
    poolPtr->pool_config.buf_size         = EM_EVENT_WG_BUF_SIZE;
    poolPtr->coh_mode         = TI_EM_COH_MODE_ON;
}

/******************************************************************************
* em_initHwConfig
******************************************************************************/
static void em_initHwConfig(ti_em_hw_config2_t* hwConfig2Ptr)
{
#ifdef TI_EM_C6678
    hwConfig2Ptr->ap_region_queue_idx= MY_EM_AP_PRIVATE_FREE_QUEUE_IDX; // 1022
    hwConfig2Ptr->cd_region_queue_idx= MY_EM_CD_PRIVATE_FREE_QUEUE_IDX; // 1023
    hwConfig2Ptr->hw_queue_base_idx  = MY_EM_HW_QUEUE_BASE_IDX;         // 1024
    hwConfig2Ptr->hw_sem_idx         = MY_EM_HW_SEM_IDX;                // 3
    hwConfig2Ptr->dma_queue_base_idx = MY_EM_DMA_QUEUE_BASE_IDX;        // 0
#endif

#ifdef TI_EM_C6634
    hwConfig2Ptr->ap_region_queue_idx= MY_EM_AP_PRIVATE_FREE_QUEUE_IDX; // 7300
    hwConfig2Ptr->cd_region_queue_idx= MY_EM_CD_PRIVATE_FREE_QUEUE_IDX; // 7301
    hwConfig2Ptr->hw_queue_base_idx  = MY_EM_HW_QUEUE_BASE_IDX;         // 6656
    hwConfig2Ptr->hw_sem_idx         = MY_EM_HW_SEM_IDX;                // 3
    hwConfig2Ptr->dma_queue_base_idx = MY_EM_DMA_QUEUE_BASE_IDX;        // 16
#endif
}


/******************************************************************************
* em_initGlobal
******************************************************************************/
int em_initGlobal(void)
{
    ti_em_pool_config2_t lvPoolConfig2Tbl[EM_POOL_NUM];
    ti_em_hw_config2_t lvHwConfig2;

    em_initPoolConfig(lvPoolConfig2Tbl); // set pools init parameters
    em_initHwConfig(&lvHwConfig2);       // set OpenEM init parameters

    /*-------------------------------------------------------------------------
    * OpenEM global initialization
    *------------------------------------------------------------------------*/
    // Get memory regions used by OpenMP
    Qmss_MemRegInfo tomp_qmssMemRegions[TOMP_NUM_QMSS_MEMORY_REGIONS];
    int tomp_qmssMemRegionCount;
    __TI_omp_get_qmss_memory_regions(tomp_qmssMemRegions, 
                                     &tomp_qmssMemRegionCount);

    if (ti_em_init_global2(tomp_qmssMemRegionCount, tomp_qmssMemRegions, 
                0, NULL,
                EM_POOL_NUM, lvPoolConfig2Tbl,
                lvHwConfig2) != EM_OK) return RETURN_FAIL;

    // Update OpenMP runtime with region allocation
    __TI_omp_update_qmss_memory_regions(tomp_qmssMemRegions, 
                                        tomp_qmssMemRegionCount);
    return RETURN_OK;
}

/******************************************************************************
* em_initLocal
******************************************************************************/
int em_initLocal(void)
    { return (ti_em_init_local2() == EM_OK) ? RETURN_OK : RETURN_FAIL; }

em_status_t eoStartDefault(void* eo_ctxt_ptr, em_eo_t eo_hdl) { return EM_OK; }
em_status_t eoStopDefault(void* eo_ctxt_ptr, em_eo_t eo_hdl)  { return EM_OK; }

/******************************************************************************
* em_initQueues
******************************************************************************/
int em_initQueues(void)
{
    EM_EO_kernel_complete = em_eo_create("Kernel Complete EO",
                                         eoStartDefault, NULL,
                                         eoStopDefault, NULL,
                                         service_kernel_complete, NULL);

    if (EM_EO_kernel_complete == EM_EO_UNDEF) return RETURN_FAIL;

    /*-------------------------------------------------------------------------
    * Create a queue group and notify Queue for each core
    *------------------------------------------------------------------------*/
    char *QGname[] = {"QG0", "QG1", "QG2", "QG3", "QG4", "QG5", "QG6", "QG7"};
    int core;
    for (core = 0; core < NUM_CORES; ++core)
    {
        em_core_mask_t mask; 
        em_core_mask_zero(&mask);
        em_core_mask_set(core, &mask);

        EM_QGRP_core[core] = em_queue_group_create(QGname[core], &mask, 0,NULL);

        if (EM_QGRP_core[core] == EM_QUEUE_GROUP_UNDEF) return RETURN_FAIL;

        EM_Q_notify[core] = em_queue_create("Notify Queue",
                                       EM_QUEUE_TYPE_PARALLEL,
                                       EM_QUEUE_PRIO_HIGHEST,
                                       EM_QGRP_core[core]);

        if (EM_Q_notify[core] == EM_QUEUE_UNDEF) return RETURN_FAIL;

        if (em_eo_add_queue(EM_EO_kernel_complete, EM_Q_notify[core]) != EM_OK)
              return RETURN_FAIL;
    }

    /*-------------------------------------------------------------------------
    * Start the kernel_complete execution Object
    *------------------------------------------------------------------------*/
    if (em_eo_start(EM_EO_kernel_complete, NULL, 0, NULL) != EM_OK)
        return RETURN_FAIL;

    /*-------------------------------------------------------------------------
    * Event groups creation
    *------------------------------------------------------------------------*/
    int i;
    for (i = 0; i < EM_EVENT_GROUP_NUM; i++)
    {
       em_hdl_event_group[i] = em_event_group_create();
       if (em_hdl_event_group[i] == EM_EVENT_GROUP_UNDEF) return RETURN_FAIL;

       /*---------------------------------------------------------------------
       * Notification events creation
       *--------------------------------------------------------------------*/
       for (core = 0; core < NUM_CORES; core++)
       {
           em_event_t event;
           do event = em_alloc(EM_EVENT_NOTIFY_BUF_SIZE,
                             TI_EM_EVENT_TYPE_PRELOAD_OFF, EM_POOL_NOTIFY_IDX);
           while (event == EM_EVENT_UNDEF);

           if (event == NULL) return RETURN_FAIL;

           em_hdl_notify_desc[i][core].event = event;
           em_hdl_notify_desc[i][core].queue = EM_Q_notify[core];
       }
    }

    /*-------------------------------------------------------------------------
    * set first free event group
    *------------------------------------------------------------------------*/
    eventGroupIdx = 0;

    em_core_mask_t  mask_wg; em_core_mask_zero(&mask_wg);

    /*-------------------------------------------------------------------------
    * Choose one of the following lines.  The set_count line will enable all
    * cores to participate in servicing worgroups.  Choose the set line to
    * limit workgroups to one core.
    *------------------------------------------------------------------------*/
#if defined(WG_1_CORE)
    em_core_mask_set(1, &mask_wg);
#else
    em_core_mask_set_count(NUM_CORES, &mask_wg);
#endif

    EM_QGRP_wg     = em_queue_group_create("QGRP_wg", &mask_wg, 0, NULL);
    EM_QUEUE(exit,      EM_QUEUE_PRIO_LOW,  EM_QUEUE_GROUP_DEFAULT);
    EM_QUEUE(workgroup, EM_QUEUE_PRIO_HIGH, EM_QGRP_wg);

    return RETURN_OK;
}
